@@ -91,12 +91,39 @@ coef.lm.rrpp <- function(object, SE = FALSE, test = FALSE, confidence = 0.95, ..
   if(SE) {
     betas <- beta.boot.iter(x, indb)
     betas <- sapply(betas, as.vector)
+    if(is.vector(betas))
+      betas <- matrix(betas, 1, length(betas))
     bd <- betas - rowMeans(betas)
+    if(is.vector(bd))
+      bd <- matrix(bd, 1, length(bd))
     se <- sqrt(rowSums(bd^2) / perms)
     if(is.matrix(coef.obs)){
       se <- matrix(se, nrow(coef.obs), ncol(coef.obs))
       dimnames(se) <- dimnames(coef.obs)
     } else names(se) <- names(coef.obs)
+    
+    if(is.matrix(se) && any(rownames(se) == "(Intercept)")){
+      rmove <- which(rownames(se) == "(Intercept)")
+      Y <- as.matrix(x$LM$Y)
+      X <- x$LM$X
+      B <- if(x$LM$gls) x$LM$gls.coefficients else x$LM$coefficients
+      result <- sapply(indb, function(x){
+        Xm <- colMeans(as.matrix(X[x, ]))
+        Ym <- colMeans(as.matrix(Y[x, ]))
+        Ym - crossprod(Xm[-rmove], B[-rmove, ])
+      })
+      
+      if(is.vector(result))  {
+        rd <- result - mean(result)
+        seint <- sqrt(sum(rd^2) / perms)
+      } else{
+        rd <- t(result) - rowMeans(result) #### left off here fix this
+        seint <- sqrt(colSums(rd^2)/perms)
+        
+      }
+      se[rmove, ] <- seint
+    }
+    
   } else se <- NULL
   
   test.ok <- (x$verbose && !x$turbo)
