@@ -101,6 +101,7 @@
 #' by high-dimensional data. Heredity. 115:357-365.
 #' 
 #' @examples 
+#' \dontrun{
 #' ### Analysis of sexual dimorphism vectors (factorial approach)
 #' data(Pupfish)
 #' fit <- lm.rrpp(coords ~ Pop * Sex, data = Pupfish, iter = 199)
@@ -146,6 +147,7 @@
 #' TP <- plot(TA, pch = 21, bg = as.numeric(motionpaths$groups),
 #' cex = 0.7, col = "gray")
 #' add.trajectories(TP, traj.pch = 21, traj.bg = 1:4)
+#' }
 
 trajectory.analysis <- function(fit, fit.null = NULL, groups, 
                                     traj.pts, pca = TRUE, print.progress = FALSE){
@@ -270,18 +272,7 @@ trajectory.analysis <- function(fit, fit.null = NULL, groups,
     dimnames(to) <- list(gl, gl)
     to
   }) else NULL
-  
-  if(is.null(Tcor)) {
-    
-    warning(
-      paste(
-        "\nUnivariate response variable detected...",
-        "\nTrajectory correlation analyses not possible.\n",
-        sep = " "),
-      noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
-    
-  }
-  
+
   # Pairwise shape differences
   
   p <- NROW(trajectories[[1]][[1]])
